/*
 *                          The Art Of
 *                      D E M O M A K I N G
 *
 *
 *                     by Alex J. Champandard
 *                          Base Sixteen
 *
 *                  Modified by Paulo Pinto (Moondevil)
 *                         for using with SDL
 *
 *                http://www.flipcode.com/demomaking
 *
 *                This file is in the public domain.
 *                      Use at your own risk.
 *
 *
 *                Compile with DJGPP:    make demo
 *
 */

#include <SDL.h>
#include <SDL_mixer.h>
#include <stdlib.h>
#include <math.h>
#include <iostream>
#include <string.h>
#include "vector.h"
#include "matrix.h"
#include "texture.h"
#include "vga.h"
#include "timer.h"
#include "midasdll.h"


using namespace std;

// this handles all the graphics
VGA *vga;

// our timing module
TIMER *timer;
long long startTime, frameCount;


// midas related stuff
MIDASplayStatus *mod_status;
MIDASmodule module;
MIDASmodulePlayHandle playHandle;

// synchronisation variables
bool bQuit = false;
float fade_coef = 0;
float flash_coef = 0;
bool bVar = false;

// temporary buffer containing the logo
unsigned char *textbuf;


// prototype so that the included routines can know how to call it
bool kbhit ();

// include each sub routine
#include "fire.h"
#include "plasma.h"
#include "roto.h"
#include "hole.h"
#include "particle.h"
#include "flubber.h"
#include "tor.h"
#include "bump.h"
#include "zoom.h"

/*
 * this is called when a sync mark Wxx is encountered in the module
 */
void MIDAS_CALL MusicCallback( unsigned syncInfo, unsigned position, unsigned row )
{
     switch (syncInfo)
     {
        case 0x01: fade_coef += 1.0; break;
        case 0x02: fade_coef -= 1.0; break;
        case 0x03: flash_coef += 0.25; break;
        case 0x04: flash_coef -= 0.25; break;
        case 0xfe: bVar = true; break;
        case 0xff: bQuit = true; break;
     }
}

long long tickCount = 0;

/*
 *
 */
void MIDAS_CALL TimerCallback(void)
{
     tickCount+=17045;
}

/*
 * Represents the kbhit() function from the DOS world
 */
bool kbhit ()
{
  bool result;

  SDL_Event event;
  if (SDL_PollEvent( &event ))
    {
      switch (event.type) {
      case SDL_QUIT:
          bQuit = true;
          break;

      case SDL_KEYDOWN:
	  result = true;
	  break;

      default:
	result = false;
      }
    }
  else
    result = false;

  return result;
}

/*
 * the main program
 */
int main()
{
// Initialize the SDL system
  if (SDL_Init( SDL_INIT_VIDEO ) < 0) {
    cerr << "Couldn't initialize SDL: " << SDL_GetError () << endl;
    exit( -1 );
  }
  atexit( SDL_Quit );

  cout << endl << "I'm wondering which came first, the chicken or the egg..." << endl;

// startup midas, should check for errors (see midas' documentation to do that)
    MIDASstartup();
    MIDASinit();

// initialise each effect
    Plasma_Init();
    Fire_Init();
    Roto_Init();
    Hole_Init();
    Flubber_Init();
    Tor_Init();
    Bump_Init();
    Zoom_Init();
 // start playing the module
    module = MIDASloadModule( "lord.xm" );
    playHandle = MIDASplayModule( module, true );
    MIDASsetMusicSyncCallback( playHandle, &MusicCallback );
    //    MIDASsetTimerCallbacks( 70000, FALSE, &TimerCallback, NULL, NULL );

    
// set mode 320x200x8
    vga = new VGA;
    Particle_Init();
// start the timer
    timer = new TIMER;
    startTime = timer->getCount();
    frameCount = 0;
    MIDASstartBackgroundPlay( 0 );
// run the main loop
    Plasma_Run();
    Fire_Run();
    Roto_Run();
    Hole_Run();
    Particle_Run();
    Flubber_Run();
    Tor_Run();
    Bump_Run();
    Zoom_Run();
    bQuit = false;
    vga->Blit( textbuf, 64000 );
// run the main loop
    while (!kbhit() && !bQuit)
    {
      Shade_Pal( 0, 255, (unsigned char)(63*flash_coef), (unsigned char)(63*flash_coef), (unsigned char)(63*flash_coef),
		 63, 63, 63 );
      vga->Update();
    }
// calculate total running time
    long long totalTime = timer->getCount() - startTime;
// and turn off the timer
    delete timer;
// deinit
    Fire_Done();
    Plasma_Done();
    Roto_Done();
    Hole_Done();
    Particle_Done();
    Flubber_Done();
    Tor_Done();
    Bump_Done();
    Zoom_Done();
// return to text mode
    delete vga;
    delete textbuf;
// shut down midas
    MIDASstopModule(playHandle);
    MIDASstopBackgroundPlay();
    MIDASfreeModule(module);
    MIDASclose();

// make sure every one knows about flipcode :)
    cout << "                             The Art Of" << endl;
    cout << "                         D E M O M A K I N G " << endl << endl;
    cout << "                       by Alex J. Champandard" << endl << endl;
    cout << "       Go to http://www.flipcode.com/demomaking for more information." << endl;
// we've finished!
    return 0;
}
